@@ -983,10 +983,17 @@ def valve_flowreorder(input_ofmsgs, use_barriers=True):
     output_ofmsgs = []
     by_kind = _partition_ofmsgs(input_ofmsgs)
 
-    # Suppress all other deletes if a global delete is present.
+    # Suppress all other relevant deletes if a global delete is present.
     delete_global_ofmsgs = by_kind.get('deleteglobal', [])
     if delete_global_ofmsgs:
-        by_kind['delete'] = []
+        global_types = []
+        for ofmsg in delete_global_ofmsgs:
+            global_types.append(type(ofmsg))
+        new_delete = []
+        for ofmsg in by_kind.get('delete', []):
+            if type(ofmsg) not in global_types:
+                new_delete.append(ofmsg)
+        by_kind['delete'] = new_delete
 
     for kind, random_order, suggest_barrier in _OFMSG_ORDER:
         ofmsgs = dedupe_ofmsgs(by_kind.get(kind, []), random_order)
