@@ -2,7 +2,7 @@
 
 # Copyright (C) 2015 Brad Cowie, Christopher Lorier and Joe Stringer.
 # Copyright (C) 2015 Research and Education Advanced Network New Zealand Ltd.
-# Copyright (C) 2015--2018 The Contributors
+# Copyright (C) 2015--2019 The Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import functools
 import hashlib
 import struct
 
@@ -72,6 +73,14 @@ class ValveTable: # pylint: disable=too-many-arguments,too-many-instance-attribu
                         '%s not configured as set field in %s' % (field, self.name))
         return valve_of.set_field(**kwds)
 
+    def set_external_forwarding_requested(self):
+        """Set field for external forwarding requested."""
+        return self.set_field(**{valve_of.EXTERNAL_FORWARDING_FIELD: valve_of.PCP_EXT_PORT_FLAG})
+
+    def set_no_external_forwarding_requested(self):
+        """Set field for no external forwarding requested."""
+        return self.set_field(**{valve_of.EXTERNAL_FORWARDING_FIELD: valve_of.PCP_NONEXT_PORT_FLAG})
+
     def set_vlan_vid(self, vlan_vid):
         """Set VLAN VID with VID_PRESENT flag set.
 
@@ -84,16 +93,17 @@ class ValveTable: # pylint: disable=too-many-arguments,too-many-instance-attribu
 
     # TODO: verify actions
     @staticmethod
-    def match(in_port=None, vlan=None, # pylint: disable=too-many-arguments
+    @functools.lru_cache(maxsize=1024)
+    def match(in_port=None, vlan=None, # pylint: disable=too-many-arguments,too-many-locals
               eth_type=None, eth_src=None, eth_dst=None, eth_dst_mask=None,
               icmpv6_type=None, nw_proto=None, nw_dst=None, metadata=None,
-              metadata_mask=None, vlan_pcp=None):
+              metadata_mask=None, vlan_pcp=None, udp_src=None, udp_dst=None):
         """Compose an OpenFlow match rule."""
         match_dict = valve_of.build_match_dict(
             in_port, vlan, eth_type, eth_src,
             eth_dst, eth_dst_mask, icmpv6_type,
             nw_proto, nw_dst, metadata, metadata_mask,
-            vlan_pcp)
+            vlan_pcp, udp_src, udp_dst)
         return valve_of.match(match_dict)
 
     def _verify_flowmod(self, flowmod):
