@@ -153,6 +153,20 @@ dps:
                 self.assertTrue(
                     valve.lacp_update_port_selection_state(port, other_valves),
                     'Port selection state not updated')
+
+                # Testing accuracy of varz port_lacp_role
+                port_labels = {
+                    'port': port.name,
+                    'port_description': port.description,
+                    'dp_name': valve.dp.name,
+                    'dp_id': '0x%x' % valve.dp.dp_id
+                }
+                lacp_role = self.get_prom('port_lacp_role', labels=port_labels, bare=True)
+                self.assertEqual(
+                    port.lacp_port_state(), lacp_role,
+                    'Port %s DP %s role %s differs from varz value %s'
+                    % (port, valve, port.lacp_port_state(), lacp_role))
+
                 if valve.dp.dp_id == 0x1:
                     self.assertEqual(
                         port.lacp_port_state(), LACP_PORT_SELECTED,
@@ -1360,9 +1374,9 @@ vlans:
                 if port.stack:
                     self.set_stack_port_up(port.number, valve)
 
-    def flood_manager_flood_ports(self, flood_manager):
+    def switch_manager_flood_ports(self, switch_manager):
         """Return list of port numbers that will be flooded to"""
-        return [port.number for port in flood_manager._stack_flood_ports()]  # pylint: disable=protected-access
+        return [port.number for port in switch_manager._stack_flood_ports()]  # pylint: disable=protected-access
 
     def route_manager_ofmsgs(self, route_manager, vlan):
         """Return ofmsgs for route stack link flooding"""
@@ -1377,7 +1391,7 @@ vlans:
         """Test intervlan flooding goes towards the root"""
         output_ports = [3]
         valve = self.valves_manager.valves[1]
-        ports = self.flood_manager_flood_ports(valve.flood_manager)
+        ports = self.switch_manager_flood_ports(valve.switch_manager)
         self.assertEqual(output_ports, ports, 'InterVLAN flooding does not match expected')
         route_manager = valve._route_manager_by_ipv.get(4, None)
         vlan = valve.dp.vlans[100]
@@ -1388,7 +1402,7 @@ vlans:
         """Test intervlan flooding goes away from the root"""
         output_ports = [3, 4]
         valve = self.valves_manager.valves[2]
-        ports = self.flood_manager_flood_ports(valve.flood_manager)
+        ports = self.switch_manager_flood_ports(valve.switch_manager)
         self.assertEqual(output_ports, ports, 'InterVLAN flooding does not match expected')
         route_manager = valve._route_manager_by_ipv.get(4, None)
         vlan = valve.dp.vlans[100]
@@ -1399,7 +1413,7 @@ vlans:
         """Test intervlan flooding only goes towards the root (s4 will get the reflection)"""
         output_ports = [3]
         valve = self.valves_manager.valves[3]
-        ports = self.flood_manager_flood_ports(valve.flood_manager)
+        ports = self.switch_manager_flood_ports(valve.switch_manager)
         self.assertEqual(output_ports, ports, 'InterVLAN flooding does not match expected')
         route_manager = valve._route_manager_by_ipv.get(4, None)
         vlan = valve.dp.vlans[100]
@@ -1410,7 +1424,7 @@ vlans:
         """Test intervlan flooding goes towards the root (through s3)"""
         output_ports = [3]
         valve = self.valves_manager.valves[4]
-        ports = self.flood_manager_flood_ports(valve.flood_manager)
+        ports = self.switch_manager_flood_ports(valve.switch_manager)
         self.assertEqual(output_ports, ports, 'InterVLAN flooding does not match expected')
         route_manager = valve._route_manager_by_ipv.get(4, None)
         vlan = valve.dp.vlans[100]
@@ -2277,7 +2291,8 @@ dps:
         self.assertIsNotNone(groupmod)
         if groupdel_exists:
             self.assertIsNotNone(groupdel)
-            self.assertTrue(groupdel.group_id, 1001)
+            if groupdel is not None:
+                self.assertTrue(groupdel.group_id, 1001)
         else:
             self.assertIsNone(groupdel)
 
